﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            if(!ProgressBarRenderer.IsSupported)
            {
                this.progressBar.BackColor = SystemColors.Control;
            }

            #region 이벤트를 설정한다.

            this.turquoiseButton.Click += turquoiseButton_Click;
            this.blueButton.Click      += blueButton_Click;
            this.greenButton.Click     += greenButton_Click;
            this.yellowButton.Click    += yellowButton_Click;
            this.redButton.Click       += redButton_Click;
            this.trackBar.Scroll       += trackBar_Scroll;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 터키색 버튼 클릭시 처리하기 - turquoiseButton_Click(sender, e)

        /// <summary>
        /// 터키색 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void turquoiseButton_Click(object sender, EventArgs e)
        {
            SetColor(Color.MediumTurquoise, Color.DarkCyan);
        }

        #endregion
        #region 파랑 버튼 클릭시 처리하기 - blueButton_Click(sender, e)

        /// <summary>
        /// 파랑 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void blueButton_Click(object sender, EventArgs e)
        {
            SetColor(Color.LightSkyBlue, Color.DodgerBlue);
        }

        #endregion
        #region 녹색 버튼 클릭시 처리하기 - greenButton_Click(sender, e)

        /// <summary>
        /// 녹색 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void greenButton_Click(object sender, EventArgs e)
        {
            SetColor(Color.LimeGreen, Color.ForestGreen);
        }

        #endregion
        #region 노랑 버튼 클릭시 처리하기 - yellowButton_Click(sender, e)

        /// <summary>
        /// 노랑 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void yellowButton_Click(object sender, EventArgs e)
        {
            SetColor(Color.Khaki, Color.Gold);
        }

        #endregion
        #region 빨강 버튼 클릭시 처리하기 - redButton_Click(sender, e)

        /// <summary>
        /// 빨강 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void redButton_Click(object sender, EventArgs e)
        {
            SetColor(Color.Red, Color.Firebrick);
        }

        #endregion
        #region 트랙바 스크롤시 처리하기 - trackBar_Scroll(sender, e)

        /// <summary>
        /// 트랙바 스크롤시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void trackBar_Scroll(object sender, EventArgs e)
        {
            this.progressBar.Value = trackBar.Value;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 색상 설정하기 - SetColor(backgroundColor, foregroungColor)

        /// <summary>
        /// 색상 설정하기
        /// </summary>
        /// <param name="backgroundColor">배경색</param>
        /// <param name="foregroungColor">전경색</param>
        private void SetColor(Color backgroundColor, Color foregroungColor)
        {
            if (ProgressBarRenderer.IsSupported)
            {
                progressBar.BackColor = backgroundColor;
                progressBar.ForeColor = foregroungColor;
            }
            else
            {
                progressBar.BackColor = SystemColors.Control;
                progressBar.ForeColor = backgroundColor;
            }
        }

        #endregion
    }
}